function [param, states] = Gibbs_update(param, Y, delta, prior)
% GIBBS_UPDATE  Perform Gibbs sampler update to parameters of GDP 
%   solera model. Model is described in construct_SSM.m and prior 
%   distribution in construct_prior.m.
%
%   PARAM = GIBBS_UPDATE(PARAM, Y, DELTA, PRIOR) updates param's PARAM
%   based on data Y, signal indicators DELTA and prior distrib PRIOR:
%     PARAM is (2*N_c+N_c^2 + N_E+N_E^2 + N_I+N_I^2)x1, see vec_parameters.m.
%     Y is NxT, contains data.
%     DELTA is NxN_c (w/N=N_E+N_I), contains signal indicators.
%     PRIOR is struct, see construct_prior.m.
%
%   [PARAM, STATES] = GIBBS_UPDATE(PARAM, Y, DELTA, PRIOR) also 
%   returns draw of states:
%     STATES is MxT (w/M=2*N_c+N_E+N_I).
%
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set some objects
N_c  = size(prior.S_X, 1);
N_E  = size(prior.S_E, 1);
N_I  = size(prior.S_I, 1);
T    = size(Y, 2);
dims = [N_c, N_E, N_I, T];
if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end

% Define function handle and options for inverse algorithm
symmetrize    = @(A) (A + A')/2;
option        = struct();
option.SYM    = true;
option.POSDEF = true;

% Map parameters
[~, ~, Sigma_X, ~, Sigma_E, ~, Sigma_I] = map_parameter(param, dims);

  
%% LATENT VARIABLES

% Form state-space representation and draw states
SSM    = construct_SSM(param, delta, dims, mu);
states = simulation_smoother(Y, SSM);


%% SIGNAL

% Construct auxiliary variables and matrices
DX = states(1:N_c, :) - states(N_c + (1:N_c), :);

% Update (mu_X, rho_X)
invSigma_X = linsolve(symmetrize(Sigma_X), eye(N_c), option);
xy_X       = [invSigma_X*sum(DX(:, 2:T), 2); ...
              zeros(N_c, 1)];
xx_X       = [(T-1)*invSigma_X, invSigma_X*diag( sum(DX(:, 1:(T-1)), 2) ); ...
              diag( sum(DX(:, 1:(T-1)), 2) )*invSigma_X, zeros(N_c)];
for t = 1:(T-1)
    xy_X(N_c+(1:N_c), :)           = xy_X(N_c+(1:N_c), :) + diag(DX(:, t))*invSigma_X*DX(:, t+1);
    xx_X(N_c+(1:N_c), N_c+(1:N_c)) = xx_X(N_c+(1:N_c), N_c+(1:N_c)) + diag(DX(:, t))*invSigma_X*diag(DX(:, t));
end
P_X_inv    = symmetrize( linsolve( kron(prior.P_X, invSigma_X) + xx_X, eye(2*N_c), option) );
m_X        = P_X_inv*(kron(prior.P_X, invSigma_X)*prior.m_X + xy_X);
beta_X     = mvnrnd(m_X, P_X_inv)';
mu_X       = beta_X(1:N_c);
rho_X      = beta_X(N_c + (1:N_c));

% Update Sigma_X
e_X     = NaN(N_c, T-1);
for t = 1:(T-1), e_X(:, t) = DX(:, t+1) - mu_X - rho_X.*DX(:, t); end
nu_X    = prior.nu_X + T-1;
S_X     = (prior.nu_X/nu_X)*prior.S_X + (1/nu_X)*e_X*(e_X');
Sigma_X = iwishrnd(nu_X*S_X, nu_X);


%% MEASUREMENT ERROR IN GDE

% Construct auxiliary variables and matrices
vE = states((2*N_c) + (1:N_E), :);

% Update rho_E
invSigma_E = linsolve(symmetrize(Sigma_E), eye(N_E), option);
xy_E       = zeros(N_E, 1);
xx_E       = zeros(N_E);
for t = 1:(T-1)
    xy_E = xy_E + diag(vE(:, t))*invSigma_E*vE(:, t+1);
    xx_E = xx_E + diag(vE(:, t))*invSigma_E*diag(vE(:, t));
end
P_E_inv    = symmetrize( linsolve(kron(prior.P_E, invSigma_E) + xx_E, eye(N_E), option) );
m_E        = P_E_inv*(kron(prior.P_E, invSigma_E)*prior.m_E + xy_E);
rho_E      = mvnrnd(m_E, P_E_inv)';

% Update Sigma_E
e_E     = NaN(N_E, T-1);
for t = 1:(T-1), e_E(:, t) = vE(:, t+1) - rho_E.*vE(:, t); end
nu_E    = prior.nu_E + T-1;
S_E     = (prior.nu_E/nu_E)*prior.S_E + (1/nu_E)*e_E*(e_E');
Sigma_E = iwishrnd(nu_E*S_E, nu_E);


%% MEASUREMENT ERROR IN GDI

% Construct auxiliary variables
vI = states((2*N_c+N_E) + (1:N_I), :);

% Update rho_I
invSigma_I = linsolve(symmetrize(Sigma_I), eye(N_I), option);
xy_I       = zeros(N_I, 1);    
xx_I       = zeros(N_I);
for t = 1:(T-1)
    xy_I = xy_I + diag(vI(:, t))*invSigma_I*vI(:, t+1);
    xx_I = xx_I + diag(vI(:, t))*invSigma_I*diag(vI(:, t));
end
P_I_inv    = symmetrize( linsolve(kron(prior.P_I, invSigma_I) + xx_I, eye(N_I), option) );
m_I        = P_I_inv*(kron(prior.P_I, invSigma_I)*prior.m_I + xy_I);
rho_I      = mvnrnd(m_I, P_I_inv)';

% Update Sigma_I
e_I     = NaN(N_I, T-1);
for t = 1:(T-1), e_I(:, t) = vI(:, t+1) - rho_I.*vI(:, t); end
nu_I    = prior.nu_I + T-1;
S_I     = (prior.nu_I/nu_I)*prior.S_I + (1/nu_I)*e_I*(e_I');
Sigma_I = iwishrnd(nu_I*S_I, nu_I);


%% OUTPUT

% Vectorize parameters
param = vec_parameter(mu_X, rho_X, Sigma_X, rho_E, Sigma_E, rho_I, Sigma_I);


end