function [param, states] = Gibbs_update(param, Y, delta, prior)
% GIBBS_UPDATE  Perform Gibbs sampler update to parameters of GDP 
%   solera model. Model is described in construct_SSM.m and prior 
%   distribution in construct_prior.m.
%
%   PARAM = GIBBS_UPDATE(PARAM, Y, DELTA, PRIOR) updates param's PARAM
%   based on data Y, signal indicators DELTA and prior distrib PRIOR:
%     PARAM is (4*N_c + 3*N_E + 3*N_I)x1, see vec_parameters.m.
%     Y is NxT, contains data.
%     DELTA is NxN_c (w/N=N_E+N_I), contains signal indicators.
%     PRIOR is struct, see construct_prior.m.
%
%   [PARAM, STATES] = GIBBS_UPDATE(PARAM, Y, DELTA, PRIOR) also 
%   returns draw of states:
%     STATES is MxT (w/M=N_c+N_E+N_I+3).
%
%   Version: 2020 Aug 06 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set some objects
N_c  = size(prior.s2_X, 1);
N_E  = size(prior.s2_E, 1);
N_I  = size(prior.s2_I, 1);
T    = size(Y, 2);
dims = [N_c, N_E, N_I, T];
if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end

% Define function handle and options for inverse algorithm
symmetrize    = @(A) (A + A')/2;
option        = struct();
option.SYM    = true;
option.POSDEF = true;

% Map parameters
[~, ~, ~, sigma_X, ~, ~, sigma_E, ~, ~, sigma_I] = map_parameter(param, dims);

  
%% LATENT VARIABLES

% Form state-space representation and draw states
SSM    = construct_SSM(param, delta, dims, mu);
states = simulation_smoother(Y, SSM);


%% SIGNAL

% Construct auxiliary variables and matrices
DX   = states(1:N_c, :) - states(N_c + (1:N_c), :);
fX   = states(2*N_c+1, :);
MatX = eye(N_c) - (1/(1+N_c))*ones(N_c);

% Update (mu_X, rho_X, lambda_X)
xy_X     = [sum(DX(:, 2:T), 2); ...
            sum(DX(:, 2:T).*DX(:, 1:(T-1)), 2); ...
            sum(DX(:, 2:T).*fX(2:T), 2)];
xx_X     = [(T-1)*eye(N_c),                 diag( sum(DX(:, 1:(T-1)), 2) ),          sum(fX(2:T))*eye(N_c); ...
            diag( sum(DX(:, 1:(T-1)), 2) ), diag( sum(DX(:, 1:(T-1)).^2, 2) ),       diag( sum(DX(:, 1:(T-1)).*fX(2:T), 2) ); ...
            sum(fX(2:T))*eye(N_c),          diag( sum(DX(:, 1:(T-1)).*fX(2:T), 2) ), sum(fX(2:T).^2)*eye(N_c)];
P_X_inv  = symmetrize( linsolve(symmetrize( kron(prior.P_X, MatX) + xx_X ), eye(3*N_c), option) );
m_X      = P_X_inv*(kron(prior.P_X, MatX)*prior.m_X + xy_X);
beta_X   = mvnrnd(m_X, symmetrize( kron(eye(3), diag(sigma_X))*P_X_inv*kron(eye(3), diag(sigma_X)) ) )';
mu_X     = beta_X(1:N_c);
rho_X    = beta_X(N_c + (1:N_c));
lambda_X = beta_X(2*N_c + (1:N_c));
sign_X   = sign(sum(lambda_X(:)));
lambda_X = sign_X*lambda_X;

% Update sigma_X
e_X     = NaN(N_c, T-1);
for t = 1:(T-1), e_X(:, t) = DX(:, t+1) - mu_X - rho_X.*DX(:, t) - lambda_X*sign_X*fX(t+1); end
nu_X    = prior.nu_X + T-1;
s2_X    = (prior.nu_X/nu_X)*prior.s2_X + (1/nu_X)*sum(e_X.^2, 2);
sigma_X = 1./sqrt(gamrnd(nu_X/2, 2./(nu_X*s2_X))); 


%% MEASUREMENT ERROR IN GDE

% Construct auxiliary variables and matrices
vE   = states((2*N_c+1) + (1:N_E), :);
fE   = states(2*N_c+1+N_E+1, :);
MatE = eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E);

% Update (rho_E, lambda_E)
xy_E     = [sum(vE(:, 2:T).*vE(:, 1:(T-1)), 2); ...
            sum(vE(:, 2:T).*fE(2:T), 2)];    
xx_E     = [diag( sum(vE(:, 1:(T-1)).^2, 2) ),       diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ); ...
            diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ), sum(fE(2:T).^2)*eye(N_E)];
P_E_inv  = symmetrize( linsolve(symmetrize(kron(prior.P_E, MatE) + xx_E), eye(2*N_E), option) );
m_E      = P_E_inv*(kron(prior.P_E, MatE)*prior.m_E + xy_E);
beta_E   = mvnrnd(m_E, symmetrize( kron(eye(2), diag(sigma_E))*P_E_inv*kron(eye(2), diag(sigma_E)) ))';
rho_E    = beta_E(1:N_E);
lambda_E = beta_E(N_E + (1:N_E));
sign_E   = sign(sum(lambda_E(:)));
lambda_E = sign_E*lambda_E;

% Update sigma_E
e_E     = NaN(N_E, T-1);
for t = 1:(T-1), e_E(:, t) = vE(:, t+1) - rho_E.*vE(:, t) - lambda_E*sign_E*fE(t+1); end
nu_E    = prior.nu_E + T-1;
s2_E    = (prior.nu_E/nu_E)*prior.s2_E + (1/nu_E)*sum(e_E.^2, 2);
sigma_E = 1./sqrt(gamrnd(nu_E/2, 2./(nu_E*s2_E))); 


%% MEASUREMENT ERROR IN GDI

% Construct auxiliary variables
vI   = states((2*N_c+1+N_E+1) + (1:N_I), :);
fI   = states(2*N_c+1+N_E+1+N_I+1, :);
MatI = eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I);

% Update (rho_I, lambda_I)
xy_I     = [sum(vI(:, 2:T).*vI(:, 1:(T-1)), 2); ...
            sum(vI(:, 2:T).*fI(2:T), 2)];
xx_I     = [diag( sum(vI(:, 1:(T-1)).^2, 2) ),       diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ); ...
            diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ), sum(fI(2:T).^2)*eye(N_I)];
P_I_inv  = symmetrize( linsolve(symmetrize(kron(prior.P_I, MatI) + xx_I), eye(2*N_I), option) );
m_I      = P_I_inv*(kron(prior.P_I, MatI)*prior.m_I + xy_I);
beta_I   = mvnrnd(m_I, symmetrize( kron(eye(2), diag(sigma_I))*P_I_inv*kron(eye(2), diag(sigma_I)) ))';
rho_I    = beta_I(1:N_I);
lambda_I = beta_I(N_I + (1:N_I));
sign_I   = sign(sum(lambda_I(:)));
lambda_I = sign_I*lambda_I;

% Update sigma_I
e_I     = NaN(N_I, T-1);
for t = 1:(T-1), e_I(:, t) = vI(:, t+1) - rho_I.*vI(:, t) - lambda_I*sign_I*fI(t+1); end
nu_I    = prior.nu_I + T-1;
s2_I    = (prior.nu_I/nu_I)*prior.s2_I + (1/nu_I)*sum(e_I.^2, 2);
sigma_I = 1./sqrt(gamrnd(nu_I/2, 2./(nu_I*s2_I)));


%% OUTPUT

% Vectorize parameters
param = vec_parameter(mu_X, rho_X, lambda_X, sigma_X, ...
    rho_E, lambda_E, sigma_E, rho_I, lambda_I, sigma_I);


end