function L = log_posterior(param, states, prior)
% LOG_POSTERIOR Compute joint log-density of parameters (posterior). 
% 
%   L = LOG_POSTERIOR(PARAM, STATE, PRIOR) returns log-density value L for
%   a vector of parameters PARAM, state variables STATES and prior PRIOR. 
% 
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c  = size(prior.S_X, 1);
N_E  = size(prior.S_E, 1);
N_I  = size(prior.S_I, 1);
T    = size(states, 2);
dims = [N_c, N_E, N_I, T];

% Extract parameters
[mu_X, rho_X, Sigma_X, ...
       rho_E, Sigma_E, ...
       rho_I, Sigma_I] = map_parameter(param, dims);

% Define function handle and options for inverse algorithm
symmetrize    = @(A) (A + A')/2;
option        = struct();
option.SYM    = true;
option.POSDEF = true;

% Preallocate log-density
L = 0;

%%% Compute log-density of parameters of signal
beta_X     = [mu_X; rho_X(:)];
invSigma_X = linsolve(symmetrize(Sigma_X), eye(N_c), option);
DX         = states(1:N_c, :) - states(N_c + (1:N_c), :);
yx_X       = invSigma_X*DX(:, 2:T)*([ones(1, T-1); DX(:, 1:(T-1))]');
xx_X       = [ones(1, T-1); DX(:, 1:(T-1))]*([ones(1, T-1); DX(:, 1:(T-1))]');
P_X_inv    = symmetrize( linsolve(symmetrize( kron(prior.P_X, invSigma_X) + kron(xx_X, invSigma_X) ), eye((1+N_c)*N_c), option) );
m_X        = P_X_inv*(kron(prior.P_X, invSigma_X)*prior.m_X + yx_X(:));
e_X        = NaN(N_c, T-1);
for t = 1:(T-1), e_X(:, t) = DX(:, t+1) - mu_X - rho_X*DX(:, t); end
nu_X       = prior.nu_X + T-1;
S_X        = (prior.nu_X/nu_X)*prior.S_X + (1/nu_X)*e_X*(e_X');
L          = L + iwishlogpdf(Sigma_X, nu_X*S_X, nu_X); 
L          = L + mvnlogpdf(beta_X, m_X, P_X_inv);

%%% Compute log-density of parameters of measurement error in GDE
invSigma_E = linsolve(symmetrize(Sigma_E), eye(N_E), option);
vE         = states((2*N_c) + (1:N_E), :);
xy_E       = zeros(N_E, 1);
xx_E       = zeros(N_E);
for t = 1:(T-1)
    xy_E = xy_E + diag(vE(:, t))*invSigma_E*vE(:, t+1);
    xx_E = xx_E + diag(vE(:, t))*invSigma_E*diag(vE(:, t));
end
P_E_inv    = symmetrize( linsolve(kron(prior.P_E, invSigma_E) + xx_E, eye(N_E), option) );
m_E        = P_E_inv*(kron(prior.P_E, invSigma_E)*prior.m_E + xy_E);
e_E        = NaN(N_E, T-1);
for t = 1:(T-1), e_E(:, t) = vE(:, t+1) - rho_E.*vE(:, t); end
nu_E       = prior.nu_E + T-1;
S_E        = (prior.nu_E/nu_E)*prior.S_E + (1/nu_E)*e_E*(e_E');
L          = L + iwishlogpdf(Sigma_E, nu_E*S_E, nu_E);
L          = L + mvnlogpdf(rho_E, m_E, P_E_inv);

%%% Compute log-density of parameters of measurement error in GDI
invSigma_I = linsolve(symmetrize(Sigma_I), eye(N_I), option);
vI         = states((2*N_c+N_E) + (1:N_I), :);
xy_I       = zeros(N_I, 1);    
xx_I       = zeros(N_I);
for t = 1:(T-1)
    xy_I = xy_I + diag(vI(:, t))*invSigma_I*vI(:, t+1);
    xx_I = xx_I + diag(vI(:, t))*invSigma_I*diag(vI(:, t));
end
P_I_inv    = symmetrize( linsolve(kron(prior.P_I, invSigma_I) + xx_I, eye(N_I), option) );
m_I        = P_I_inv*(kron(prior.P_I, invSigma_I)*prior.m_I + xy_I);
e_I        = NaN(N_I, T-1);
for t = 1:(T-1), e_I(:, t) = vI(:, t+1) - rho_I.*vI(:, t); end
nu_I       = prior.nu_I + T-1;
S_I        = (prior.nu_I/nu_I)*prior.S_I + (1/nu_I)*e_I*(e_I');
L          = L + iwishlogpdf(Sigma_I, nu_I*S_I, nu_I);
L          = L + mvnlogpdf(rho_I, m_I, P_I_inv);

end


%% LOCAL FUNCTIONS

function p = iwishlogpdf(S, Tau, df)
% IWISHLOGPDF Evaluates the log PDF of the inverse Wishart distribution.
%
%   p = iwishlogpdf(S, T, df) returns the log PDF of the inverse Wishart
%   distribution at the given matrix S, with T as the inverse covariance
%   matrix and df as the degrees of freedom.
%
%   Input:
%       - S: The matrix at which to evaluate the PDF.
%       - Tau: The inverse covariance matrix.
%       - df: Degrees of freedom.
%
%   Output:
%       - p: The PDF value at the given matrix S.

% Check if the inputs are valid
if ~(isnumeric(S) && isnumeric(Tau) && isscalar(df) && df > 0)
    error('Invalid input arguments.');
end

% Get the dimensions of the matrices
[n, ~] = size(S);
[m, ~] = size(Tau);

% Check if the matrices are square and have compatible dimensions
if ~isequal(size(S), size(Tau)) || n ~= m || ~isposdef(S) || ~isposdef(Tau)
    error('Invalid matrix dimensions or non-positive definite matrix.');
end

% Check if the inputs are non-negative
if any(df < n-1)
    error('Invalid input arguments. Degrees of freedom must be greater than n-1.');
end

% Calculate the constant term in the PDF formula
C = (df/2)*log(det(Tau)) - (df*n/2)*log(2) - multivar_gammaln(df/2, n);

% Evaluate the PDF
p = C - ((df+n+1)/2)*log(det(S)) - trace(Tau/S)/2;

end

function result = multivar_gammaln(x, p)
    % Compute the multivariate gamma function using the product formula
    result = (p*(p-1)/4)*log(pi) + sum(gammaln(x + (1-(1:p))/2));
end

function result = isposdef(A)
    % Helper function to check if a matrix is positive definite
    result = all(eig(A) > 0);
end

function p = mvnlogpdf(x, mu, Sigma)
    x  = x(:);
    mu = mu(:);
    n  = size(x(:), 1);
    p  = -(n/2)*log(2*pi) - (1/2)*log(det(Sigma)) - (x-mu)'*(Sigma\(x-mu))/2;
end