function L = log_prior(param, prior)
% LOG_PRIOR Compute joint log-density of parameters (prior). 
% 
%   L = LOG_PRIOR(PARAM, PRIOR) returns log-density value L for a vector
%   of parameters PARAM and prior PRIOR.
% 
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set function handles
N_c        = size(prior.S_X, 1);
N_E        = size(prior.S_E, 1);
N_I        = size(prior.S_I, 1);
dims       = [N_c, N_E, N_I];
symmetrize = @(A) (A + A')/2;

% Extract parameters
[mu_X, rho_X, Sigma_X, ...
       rho_E, Sigma_E, ...
       rho_I, Sigma_I] = map_parameter(param, dims);

% Preallocate log-density
L = 0;

% Compute log-density of parameters of signal
beta_X = [mu_X; rho_X(:)];
L      = L + iwishlogpdf(Sigma_X, prior.nu_X*prior.S_X, prior.nu_X); 
L      = L + mvnlogpdf(beta_X, prior.m_X, symmetrize( kron(pinv(prior.P_X), Sigma_X) ));

% Compute log-density of parameters of measurement error in GDE
L = L + iwishlogpdf(Sigma_E, prior.nu_E*prior.S_E, prior.nu_E);
L = L + mvnlogpdf(rho_E, prior.m_E, symmetrize( kron(pinv(prior.P_E), Sigma_E) ));

% Compute log-density of parameters of measurement error in GDI
L = L + iwishlogpdf(Sigma_I, prior.nu_I*prior.S_I, prior.nu_I);
L = L + mvnlogpdf(rho_I, prior.m_I, symmetrize( kron(pinv(prior.P_I), Sigma_I) ));

end


%% LOCAL FUNCTIONS

function p = iwishlogpdf(S, Tau, df)
% IWISHLOGPDF Evaluates the log PDF of the inverse Wishart distribution.
%
%   p = iwishlogpdf(S, T, df) returns the log PDF of the inverse Wishart
%   distribution at the given matrix S, with T as the inverse covariance
%   matrix and df as the degrees of freedom.
%
%   Input:
%       - S: The matrix at which to evaluate the PDF.
%       - Tau: The inverse covariance matrix.
%       - df: Degrees of freedom.
%
%   Output:
%       - p: The PDF value at the given matrix S.

% Check if the inputs are valid
if ~(isnumeric(S) && isnumeric(Tau) && isscalar(df) && df > 0)
    error('Invalid input arguments.');
end

% Get the dimensions of the matrices
[n, ~] = size(S);
[m, ~] = size(Tau);

% Check if the matrices are square and have compatible dimensions
if ~isequal(size(S), size(Tau)) || n ~= m || ~isposdef(S) || ~isposdef(Tau)
    error('Invalid matrix dimensions or non-positive definite matrix.');
end

% Check if the inputs are non-negative
if any(df < n-1)
    error('Invalid input arguments. Degrees of freedom must be greater than n-1.');
end

% Calculate the constant term in the PDF formula
C = (df/2)*log(det(Tau)) - (df*n/2)*log(2) - multivar_gammaln(df/2, n);

% Evaluate the PDF
p = C - ((df+n+1)/2)*log(det(S)) - trace(Tau/S)/2;

end

function result = multivar_gammaln(x, p)
    % Compute the multivariate gamma function using the product formula
    result = (p*(p-1)/4)*log(pi) + sum(gammaln(x + (1-(1:p))/2));
end

function result = isposdef(A)
    % Helper function to check if a matrix is positive definite
    result = all(eig(A) > 0);
end

function p = mvnlogpdf(x, mu, Sigma)
    x  = x(:);
    mu = mu(:);
    n  = size(x(:), 1);
    p  = -(n/2)*log(2*pi) - (1/2)*log(det(Sigma)) - (x-mu)'*(Sigma\(x-mu))/2;
end