function description = name_parameter(dims)
% NAME_PARAMETER  Create description for individual parameters of GDP 
%   solera model.
% 
%   DESCRIPTION = NAME_PARAMETER(DIMS) returns description using
%   par's shaped with dimensions DIMS = [N_C, N_E, N_I].
%
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);

% Set dimensions
n_param_r = [N_c, N_c, N_c, N_E, N_E, N_I, N_I];
n_param_c = [1,   N_c, N_c, 1,   N_E, 1,   N_I];
n_param   = n_param_r .* n_param_c;
var_param = [1,   1,   1,   2,   2,   3,   3];

% Set numbering strings
var_name    = cell(3, 1);
var_name{1} = cell(N_c, 1);
for i = 1:N_c, var_name{1}{i} = num2str(i); end
var_name{2} = cell(N_E, 1);
for i = 1:(N_E-N_c), var_name{2}{i}         = ['nc' num2str(i)]; end
for i = 1:N_c,       var_name{2}{N_E-N_c+i} = ['c' num2str(i)]; end
var_name{3} = cell(N_I, 1);
for i = 1:(N_I-N_c), var_name{3}{i}         = ['nc' num2str(i)]; end
for i = 1:N_c,       var_name{3}{N_I-N_c+i} = ['c' num2str(i)]; end

% Store number of parameters and names in parameter groups
description                  = struct();
description.n_param          = n_param;
description.param_groupnames = {'mu_X', 'rho_X',   'Sigma_X', ...
                                        'rho_GDE', 'Sigma_GDE', ...
                                        'rho_GDI', 'Sigma_GDI'};

% Form TEX-compatible parameter names vector for figures
TEX_groupnames = {'\mu_x', '\rho_x',     '\Sigma_x', ...
                           '\rho_{GDE}', '\Sigma_{GDE}', ...
                           '\rho_{GDI}', '\Sigma_{GDI}'};
param_names = cell(sum(n_param, 2), 1);
i_tmp       = 1;
for i_paramunit = 1:length(TEX_groupnames)
    varname_tmp = var_name{var_param(i_paramunit)};
    if (n_param_c(i_paramunit) == 1)
        for i_r = 1:n_param_r(i_paramunit)
            param_names{i_tmp} = sprintf('%s^{(%s)}', TEX_groupnames{i_paramunit}, varname_tmp{i_r});
            i_tmp              = i_tmp + 1;
        end
    else
        for i_c = 1:n_param_c(i_paramunit)
            for i_r = 1:n_param_r(i_paramunit)
                param_names{i_tmp} = sprintf('%s^{(%s, %s)}', TEX_groupnames{i_paramunit}, varname_tmp{i_r}, varname_tmp{i_c});
                i_tmp              = i_tmp + 1;
            end
        end
    end
end
description.param_names_fig = param_names;

% Form TEX-compatible parameter names vector for table
param_names = cell(sum(n_param, 2), 1);
i_tmp       = 1;
for i_paramunit = 1:length(TEX_groupnames)
    varname_tmp = var_name{var_param(i_paramunit)};
    if (n_param_c(i_paramunit) == 1)
        for i_r = 1:n_param_r(i_paramunit)
            param_names{i_tmp} = sprintf('${%s}^{(%s)}$', TEX_groupnames{i_paramunit}, varname_tmp{i_r});
            i_tmp              = i_tmp + 1;
        end
    else
        for i_c = 1:n_param_c(i_paramunit)
            for i_r = 1:n_param_r(i_paramunit)
                param_names{i_tmp} = sprintf('${%s}^{(%s, %s)}$', TEX_groupnames{i_paramunit}, varname_tmp{i_r}, varname_tmp{i_c});
                i_tmp              = i_tmp + 1;
            end
        end
    end
end
description.param_names_tab = param_names;

end