function SSM = construct_SSM(param, delta, dims, mu)
% CONSTRUCT_SSM  Construct state-space form of GDP solera model:
%   Y_(Et)     = delta_(Et) * x_t + v_(Et),
%   Y_(It)     = delta_(It) * x_t + v_(It),
%   Dx_(t+1)   = mu_X + rho_X * Dx_t + lambda_X * eta_(Xt) + sigma_X .* eps_(Xt),
%   v_(E, t+1) = rho_E .* v_(Et) + lambda_E * eta_(Et) + sigma_E .* eps_(Et),
%   v_(I, t+1) = rho_I .* v_(It) + lambda_I * eta_(It) + sigma_I .* eps_(It).
%
%   Initial condition is distributed as
%   (x_1, v_(E1), v_(I1)) ~ N(mu_1, Sigma_1).
%
%   Shocks and common factors are distributed as
%   eta_(Xt) ~ N(0, 1),
%   eps_(Xt) ~ N(0_(N_cx1), I_(N_c)),
%   eta_(Et) ~ N(0, 1),
%   eps_(Et) ~ N(0_(N_Ex1), I_(N_E)),
%   eta_(It) ~ N(0, 1),
%   eps_(It) ~ N(0_(N_Ix1), I_(N_I)).
%
%   SSM = CONSTRUCT_SSM(PARAM, DELTA, DIMS, MU, GAM) returns state-space
%   model matrices based on parameters PARAM, signal indicators DELTA,
%   dimensions DIMS, initial mean of signals MU and correlation of
%   measurement errors GAM:
%     PARAM is (4*N_c + 3*N_E + 3*N_I)x1, see vec_parameters.m.
%     DELTA is NxN_c (w/N=N_E+N_I), contains signal indicators delta_(Et) 
%     and delta_(It).
%     DIMS = [N_c, N_E, N_I, T], MU is N_cx1 and GAM is scalar
%     SSM is struct:
%     - SSM.H is NxMxT (w/M=2*N_c+N_E+N_I+3), time varying as delta.
%     - SSM.C is Mx1.
%     - SSM.F is MxM.
%     - SSM.G is MxK (w/K=N_c+N_E+N_I+3).
%     - SSM.mu_1 is Mx1.
%     - SSM.Sigma_1 is MxM.
%
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);
T   = dims(4);
N   = N_E + N_I;

% Control correlation between common components of signal and noises
if ((nargin < 4) || isempty(mu)),  mu = zeros(N_c, 1); end; mu(isnan(mu)) = 0;

% Extract parameters from vector
[mu_X, rho_X, lambda_X, sigma_X, ...
       rho_E, lambda_E, sigma_E, ...
       rho_I, lambda_I, sigma_I] = map_parameter(param, dims);
            
% Form state space matrices
H = NaN(N, 2*N_c+N+3, T);
for t = 1:T 
    H(:, :, t) = [delta(:, :, t), zeros(N, N_c+1), ...
                  blkdiag([eye(N_E), zeros(N_E, 1)], [eye(N_I), zeros(N_I, 1)])];
end
Sigma_eps = (1e-4)*eye(N_E+N_I);
C = [mu_X; zeros(N_c+N+3, 1)];
F = blkdiag([rho_X+eye(N_c), -rho_X,        zeros(N_c, 1); ...
             eye(N_c),       zeros(N_c),    zeros(N_c, 1); ...
             zeros(1, N_c),  zeros(1, N_c), 0            ], ...
            diag([rho_E; 0]), diag([rho_I; 0]));
G = blkdiag([diag(sigma_X), lambda_X; ...
             zeros(N_c),    zeros(N_c, 1); ...
             zeros(1, N_c), 1            ], ...
            [diag(sigma_E), lambda_E; zeros(1, N_E), 1], ...
            [diag(sigma_I), lambda_I; zeros(1, N_I), 1]);
Sigma_eta                           = eye(N_c+1+N_E+1+N_I+1);
Sigma_1 = zeros(2*N_c+N+3);
for t = 1:T, Sigma_1 = F*Sigma_1*(F') + G*(G'); end

% Pass output as struct
SSM           = struct();
SSM.H         = H;
SSM.Sigma_eps = Sigma_eps;
SSM.C         = C;
SSM.F         = F;
SSM.G         = G;
SSM.Sigma_eta = Sigma_eta;
SSM.mu_1      = [repmat(mu(:), [2, 1]); zeros(N+3, 1)];
SSM.Sigma_1   = Sigma_1; 

end