function param = simulate_prior(prior)
% SIMULATE_PRIOR Draw parameters of GDP solera model from prior distrib. 
% 
%   PARAM = SIMULATE_PRIOR(PRIOR) returns vector of parameters PARAM from
%   prior distribution PRIOR:
%     PRIOR is struct, see construct_prior.m.
%     PARAM is (3*N_c+N_c^2 + 3*N_E + 3*N_I)x1, see vec_parameters.m.
% 
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set function handles
N_c        = size(prior.s2_X, 1);
N_E        = size(prior.s2_E, 1);
N_I        = size(prior.s2_I, 1);
symmetrize = @(A) (A + A')/2;

% Draw parameters of signal
sigma_X  = 1./sqrt(gamrnd(prior.nu_X/2, 2./(prior.nu_X*prior.s2_X))); 
MatX     = diag(sigma_X) * ((eye(N_c) - (1/(1+N_c))*ones(N_c)) \ diag(sigma_X));
beta_X   = mvnrnd(prior.m_X, symmetrize( kron(pinv(prior.P_X), MatX) ))';
mu_X     = beta_X(1:N_c);
rho_X    = reshape(beta_X(N_c + (1:N_c^2)), [N_c, N_c]);
lambda_X = beta_X(N_c+N_c^2 + (1:N_c));
lambda_X = sign(sum(lambda_X(:)))*lambda_X;

% Draw parameters of measurement error in GDP
sigma_E  = 1./sqrt(gamrnd(prior.nu_E/2, 2./(prior.nu_E*prior.s2_E)));
MatE     = diag(sigma_E) * ((eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E)) \ diag(sigma_E));
beta_E   = mvnrnd(prior.m_E, symmetrize( kron(pinv(prior.P_E), MatE) ))';
rho_E    = beta_E(1:N_E);
lambda_E = beta_E(N_E + (1:N_E));
lambda_E = sign(sum(lambda_E(:)))*lambda_E;

% Draw parameters of measurement error in GDI
sigma_I  = 1./sqrt(gamrnd(prior.nu_I/2, 2./(prior.nu_I*prior.s2_I)));
MatI     = diag(sigma_I) * ((eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I)) \ diag(sigma_I));
beta_I   = mvnrnd(prior.m_I, symmetrize( kron(pinv(prior.P_I), MatI) ))';
rho_I    = beta_I(1:N_I);
lambda_I = beta_I(N_I + (1:N_I));
lambda_I = sign(sum(lambda_I(:)))*lambda_I;

% Vectorize parameters
param = vec_parameter(mu_X, rho_X, lambda_X, sigma_X, ...
                            rho_E, lambda_E, sigma_E, ...
                            rho_I, lambda_I, sigma_I);
                      
end