function [param, states] = Gibbs_update(param, Y, delta, prior)
% GIBBS_UPDATE  Perform Gibbs sampler update to parameters of GDP 
%   solera model. Model is described in construct_SSM.m and prior 
%   distribution in construct_prior.m.
%
%   PARAM = GIBBS_UPDATE(PARAM, Y, DELTA, PRIOR) updates param's PARAM
%   based on data Y, signal indicators DELTA and prior distrib PRIOR:
%     PARAM is (9*N_c + 3*N_E + 3*N_I)x1, see vec_parameters.m.
%     Y is NxT, contains data.
%     DELTA is NxN_c (w/N=N_E+N_I), contains signal indicators.
%     PRIOR is struct, see construct_prior.m.
%
%   [PARAM, STATES] = GIBBS_UPDATE(PARAM, Y, DELTA, PRIOR) also 
%   returns draw of states:
%     STATES is MxT (w/M=2*N_c+N_E+N_I+4).
%
%   Version: 2023 Jul 11 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set some objects
N_c  = size(prior.s2_T, 1);
N_E  = size(prior.s2_E, 1);
N_I  = size(prior.s2_I, 1);
T    = size(Y, 2);
dims = [N_c, N_E, N_I, T];
if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end

% Define function handle and options for inverse algorithm
symmetrize    = @(A) (A + A')/2;
option        = struct();
option.SYM    = true;
option.POSDEF = true;

% Map parameters
[mu_T, lambda_T, sigma_T, ~, ~, sigma_C, ~, ~, sigma_E, ~, ~, sigma_I] = map_parameter(param, dims);

  
%% LATENT VARIABLES

% Form state-space representation and draw states
SSM    = construct_SSM(param, delta, dims, mu);
states = simulation_smoother(Y, SSM);


%% SIGNAL: CYCLE

% Construct idiosyncratic errors from trend
DT     = states(1:N_c, 2:T) - states(1:N_c, 1:(T-1));
fT     = states(N_c+1, 2:T);
sign_T = sign(sum(lambda_T));
e_T    = NaN(N_c, T-1);
for t = 1:(T-1), e_T(:, t) = DT(:, t) - mu_T - lambda_T*sign_T*fT(t); end
e_T = e_T./sigma_T;

% Construct auxiliary variables and matrices
C_0  = states(N_c+1+(1:N_c), 2:T);
C_1  = states(N_c+1+(1:N_c), 1:(T-1));
C_2  = states(2*N_c+1+(1:N_c), 1:(T-1));
fC   = states(3*N_c+2, 2:T);
MatC = eye(N_c) - (1/(1+N_c))*ones(N_c);

% Update (rho_C, lambda_C, sigma_C(:, 1))
xy_C     = [sum(C_0.*C_1, 2); ...
            sum(C_0.*C_2, 2); ...
            sum(C_0.*fT, 2); ...
            sum(C_0.*fC, 2); ...
            sum(C_0.*e_T, 2)];
xx_C     = [diag(sum(C_1.^2, 2)), diag(sum(C_1.*C_2, 2)), diag(sum(C_1.*fT, 2)), diag(sum(C_1.*fC, 2)), diag(sum(C_1.*e_T, 2)); ...
            diag(sum(C_2.*C_1, 2)), diag(sum(C_2.^2, 2)), diag(sum(C_2.*fT, 2)), diag(sum(C_2.*fC, 2)), diag(sum(C_2.*e_T, 2)); ...
            diag(sum(fT.*C_1, 2)), diag(sum(fT.*C_2, 2)), sum(fT.^2, 2)*eye(N_c), sum(fT.*fC, 2)*eye(N_c), diag(sum(fT.*e_T, 2)); ...
            diag(sum(fC.*C_1, 2)), diag(sum(fC.*C_2, 2)), sum(fC.*fT, 2)*eye(N_c), sum(fC.^2, 2)*eye(N_c), diag(sum(fC.*e_T, 2)); ...
            diag(sum(e_T.*C_1, 2)), diag(sum(e_T.*C_2, 2)), diag(sum(e_T.*fT, 2)), diag(sum(e_T.*fC, 2)), diag(sum(e_T.^2, 2))];
P_C_inv  = symmetrize( linsolve(symmetrize( kron(prior.P_C, MatC) + xx_C ), eye(5*N_c), option) );
m_C      = P_C_inv*(kron(prior.P_C, MatC)*prior.m_C + xy_C);
beta_C   = mvnrnd(m_C, symmetrize( kron(eye(5), diag(sigma_C(:, 2)))*P_C_inv*kron(eye(5), diag(sigma_C(:, 2))) ) )';
rho_C    = reshape(beta_C(1:2*N_c), [N_c, 2]);
lambda_C = reshape(beta_C(2*N_c + (1:2*N_c)), [N_c, 2]);
sign_C   = sign(sum(lambda_C(:, 2)));
lambda_C = [sign_T*lambda_C(:, 1), sign_C*lambda_C(:, 2)];
sigma_C(:, 1) = beta_C(4*N_c + (1:N_c));

% Impose stationarity
for i_c = 1:N_c
    root_tmp = roots([1 -rho_C(i_c, 1) -rho_C(i_c, 2)]);
    root_tmp = root_tmp + min(max(0, -0.99-real(root_tmp)), 0.99-real(root_tmp));
    poly_tmp = poly(root_tmp);
    rho_C(i_c, :) = -[poly_tmp(2), poly_tmp(3)];
end

% Update sigma_C(:, 2)
e_C     = NaN(N_c, T-1);
for t = 1:(T-1), e_C(:, t) = C_0(:, t) - rho_C(:, 1).*C_1(:, t) - rho_C(:, 2).*C_2(:, t) - lambda_C(:, 1)*sign_T*fT(t) - lambda_C(:, 2)*sign_C*fC(t) - sigma_C(:, 1).*e_T(:, t); end
nu_C    = prior.nu_C + T-1;
s2_C    = (prior.nu_C/nu_C)*prior.s2_C + (1/nu_C)*sum(e_C.^2, 2);
sigma_C(:, 2) = 1./sqrt(gamrnd(nu_C/2, 2./(nu_C*s2_C))); 


%% SIGNAL: TREND

% Construct auxiliary variables and matrices
DT   = states(1:N_c, 2:T) - states(1:N_c, 1:(T-1));
fT   = states(N_c+1, 2:T);
MatT = eye(N_c) - (1/(1+N_c))*ones(N_c);

% Update (mu_T, lambda_T)
xy_T     = [sum(DT, 2); ...
            sum(DT.*fT, 2)];
xx_T     = [(T-1)*eye(N_c),   sum(fT)*eye(N_c); ...
            sum(fT)*eye(N_c), sum(fT.^2)*eye(N_c)];
P_T_inv  = symmetrize( linsolve(symmetrize( kron(prior.P_T, MatT) + xx_T ), eye(2*N_c), option) );
m_T      = P_T_inv*(kron(prior.P_T, MatT)*prior.m_T + xy_T);
beta_T   = mvnrnd(m_T, symmetrize( kron(eye(2), diag(sigma_T))*P_T_inv*kron(eye(2), diag(sigma_T)) ) )';
mu_T     = beta_T(1:N_c);
lambda_T = beta_T(N_c + (1:N_c));
sign_T   = sign(sum(lambda_T(:)));
lambda_T = sign_T*lambda_T;

% Update sigma_T
e_T     = NaN(N_c, T-1);
for t = 1:(T-1), e_T(:, t) = DT(:, t) - mu_T - lambda_T*sign_T*fT(t); end
nu_T    = prior.nu_T + T-1;
s2_T    = (prior.nu_T/nu_T)*prior.s2_T + (1/nu_T)*sum(e_T.^2, 2);
sigma_T = 1./sqrt(gamrnd(nu_T/2, 2./(nu_T*s2_T))); 


%% MEASUREMENT ERROR IN GDE

% Construct auxiliary variables and matrices
vE   = states((3*N_c+2) + (1:N_E), :);
fE   = states(3*N_c+2+N_E+1, :);
MatE = eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E);

% Update (rho_E, lambda_E)
xy_E     = [sum(vE(:, 2:T).*vE(:, 1:(T-1)), 2); ...
            sum(vE(:, 2:T).*fE(2:T), 2)];    
xx_E     = [diag( sum(vE(:, 1:(T-1)).^2, 2) ),       diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ); ...
            diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ), sum(fE(2:T).^2)*eye(N_E)];
P_E_inv  = symmetrize( linsolve(symmetrize(kron(prior.P_E, MatE) + xx_E), eye(2*N_E), option) );
m_E      = P_E_inv*(kron(prior.P_E, MatE)*prior.m_E + xy_E);
beta_E   = mvnrnd(m_E, symmetrize( kron(eye(2), diag(sigma_E))*P_E_inv*kron(eye(2), diag(sigma_E)) ))';
rho_E    = beta_E(1:N_E);
lambda_E = beta_E(N_E + (1:N_E));
sign_E   = sign(sum(lambda_E(:)));
lambda_E = sign_E*lambda_E;

% Update sigma_E
e_E     = NaN(N_E, T-1);
for t = 1:(T-1), e_E(:, t) = vE(:, t+1) - rho_E.*vE(:, t) - lambda_E*sign_E*fE(t+1); end
nu_E    = prior.nu_E + T-1;
s2_E    = (prior.nu_E/nu_E)*prior.s2_E + (1/nu_E)*sum(e_E.^2, 2);
sigma_E = 1./sqrt(gamrnd(nu_E/2, 2./(nu_E*s2_E))); 


%% MEASUREMENT ERROR IN GDI

% Construct auxiliary variables
vI   = states((3*N_c+2+N_E+1) + (1:N_I), :);
fI   = states(3*N_c+2+N_E+1+N_I+1, :);
MatI = eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I);

% Update (rho_I, lambda_I)
xy_I     = [sum(vI(:, 2:T).*vI(:, 1:(T-1)), 2); ...
            sum(vI(:, 2:T).*fI(2:T), 2)];
xx_I     = [diag( sum(vI(:, 1:(T-1)).^2, 2) ),       diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ); ...
            diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ), sum(fI(2:T).^2)*eye(N_I)];
P_I_inv  = symmetrize( linsolve(symmetrize(kron(prior.P_I, MatI) + xx_I), eye(2*N_I), option) );
m_I      = P_I_inv*(kron(prior.P_I, MatI)*prior.m_I + xy_I);
beta_I   = mvnrnd(m_I, symmetrize( kron(eye(2), diag(sigma_I))*P_I_inv*kron(eye(2), diag(sigma_I)) ))';
rho_I    = beta_I(1:N_I);
lambda_I = beta_I(N_I + (1:N_I));
sign_I   = sign(sum(lambda_I(:)));
lambda_I = sign_I*lambda_I;

% Update sigma_I
e_I     = NaN(N_I, T-1);
for t = 1:(T-1), e_I(:, t) = vI(:, t+1) - rho_I.*vI(:, t) - lambda_I*sign_I*fI(t+1); end
nu_I    = prior.nu_I + T-1;
s2_I    = (prior.nu_I/nu_I)*prior.s2_I + (1/nu_I)*sum(e_I.^2, 2);
sigma_I = 1./sqrt(gamrnd(nu_I/2, 2./(nu_I*s2_I)));


%% OUTPUT

% Vectorize parameters
param = vec_parameter(mu_T, lambda_T, sigma_T, rho_C, lambda_C, sigma_C, ...
                      rho_E, lambda_E, sigma_E, rho_I, lambda_I, sigma_I);

end