function param = simulate_prior(prior)
% SIMULATE_PRIOR Draw parameters of GDP solera model from prior distrib. 
% 
%   PARAM = SIMULATE_PRIOR(PRIOR) returns vector of parameters PARAM from
%   prior distribution PRIOR:
%     PRIOR is struct, see construct_prior.m.
%     PARAM is (9*N_c + 3*N_E + 3*N_I)x1, see vec_parameters.m.
% 
%   Version: 2023 Jul 11 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set function handles
N_c        = size(prior.s2_T, 1);
N_E        = size(prior.s2_E, 1);
N_I        = size(prior.s2_I, 1);
symmetrize = @(A) (A + A')/2;

% Draw parameters of trend in signal
sigma_T  = 1./sqrt(gamrnd(prior.nu_T/2, 2./(prior.nu_T*prior.s2_T))); 
MatT     = diag(sigma_T) * ((eye(N_c) - (1/(1+N_c))*ones(N_c)) \ diag(sigma_T));
beta_T   = mvnrnd(prior.m_T, symmetrize( kron(pinv(prior.P_T), MatT) ))';
mu_T     = beta_T(1:N_c);
lambda_T = beta_T(N_c + (1:N_c));
sign_T   = sign(sum(lambda_T(:)));
lambda_T = sign_T*lambda_T;

% Draw parameters of cycle in signal
sigma_C2 = 1./sqrt(gamrnd(prior.nu_C/2, 2./(prior.nu_C*prior.s2_C))); 
MatC     = diag(sigma_C2) * ((eye(N_c) - (1/(1+N_c))*ones(N_c)) \ diag(sigma_C2));
beta_C   = mvnrnd(prior.m_C, symmetrize( kron(pinv(prior.P_C), MatC) ))';
rho_C    = reshape(beta_C(1:2*N_c), [N_c, 2]);
lambda_C = reshape(beta_C(2*N_c + (1:2*N_c)), [N_c, 2]);
sign_C   = sign(sum(lambda_C(:, 2)));
lambda_C = [sign_T*lambda_C(:, 1), sign_C*lambda_C(:, 2)];
sigma_C  = [beta_C(4*N_c + (1:N_c)), sigma_C2];

% Impose stationarity
for i_c = 1:N_c
    root_tmp = roots([1 -rho_C(i_c, 1) -rho_C(i_c, 2)]);
    root_tmp = root_tmp + min(max(0, -0.99-real(root_tmp)), 0.99-real(root_tmp));
    poly_tmp = poly(root_tmp);
    rho_C(i_c, :) = -[poly_tmp(2), poly_tmp(3)];
end

% Draw parameters of measurement error in GDP
sigma_E  = 1./sqrt(gamrnd(prior.nu_E/2, 2./(prior.nu_E*prior.s2_E)));
MatE     = diag(sigma_E) * ((eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E)) \ diag(sigma_E));
beta_E   = mvnrnd(prior.m_E, symmetrize( kron(pinv(prior.P_E), MatE) ))';
rho_E    = beta_E(1:N_E);
lambda_E = beta_E(N_E + (1:N_E));
lambda_E = sign(sum(lambda_E(:)))*lambda_E;

% Draw parameters of measurement error in GDI
sigma_I  = 1./sqrt(gamrnd(prior.nu_I/2, 2./(prior.nu_I*prior.s2_I)));
MatI     = diag(sigma_I) * ((eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I)) \ diag(sigma_I));
beta_I   = mvnrnd(prior.m_I, symmetrize( kron(pinv(prior.P_I), MatI) ))';
rho_I    = beta_I(1:N_I);
lambda_I = beta_I(N_I + (1:N_I));
lambda_I = sign(sum(lambda_I(:)))*lambda_I;

% Vectorize parameters
param = vec_parameter(mu_T, lambda_T, sigma_T, rho_C, lambda_C, sigma_C, ...
                      rho_E, lambda_E, sigma_E, rho_I, lambda_I, sigma_I);
                      
end